from flask import Blueprint, render_template, request, redirect, url_for, flash, jsonify, send_from_directory, current_app
from flask_login import current_user, login_required
from werkzeug.utils import secure_filename
import os
from app import db
from .models import SubmissionImage, ImageHeart, FeaturedImage
from plugins.plant.models import Plant

bp = Blueprint("submission", __name__, template_folder="templates")

UPLOAD_FOLDER = "static/uploads"
ALLOWED_EXTENSIONS = {"png", "jpg", "jpeg", "gif"}

def allowed_file(filename):
    return "." in filename and filename.rsplit(".", 1)[1].lower() in ALLOWED_EXTENSIONS

@bp.route("/submissions/upload", methods=["GET", "POST"])
@login_required
def upload_submissions():
    if request.method == "POST":
        file = request.files.get("image")
        caption = request.form.get("caption")
        plant_id = request.form.get("plant_id")

        if file and allowed_file(file.filename):
            filename = secure_filename(file.filename)
            save_path = os.path.join(current_app.root_path, UPLOAD_FOLDER)
            os.makedirs(save_path, exist_ok=True)
            file.save(os.path.join(save_path, filename))

            submissions = SubmissionImage(file_url=f"{UPLOAD_FOLDER}/{filename}", caption=caption, plant_id=plant_id)
            db.session.add(submissions)
            db.session.commit()

            flash("Image uploaded successfully.", "success")
            return redirect(url_for("submissions.upload_submissions"))
        else:
            flash("Invalid file or no file uploaded.", "danger")

    return render_template("submissions/upload.html")

@bp.route("/submissions/files/<path:filename>")
def submissions_file(filename):
    return send_from_directory(os.path.join(current_app.root_path, "static/uploads"), filename)

@bp.route("/submissions/heart/<int:image_id>", methods=["POST"])
@login_required
def toggle_heart(image_id):
    existing = ImageHeart.query.filter_by(user_id=current_user.id, submission_image_id=image_id).first()
    if existing:
        db.session.delete(existing)
        db.session.commit()
        return jsonify({"status": "unhearted"})
    else:
        heart = ImageHeart(user_id=current_user.id, submission_image_id=image_id)
        db.session.add(heart)
        db.session.commit()
        return jsonify({"status": "hearted"})

@bp.route("/submissions/feature/<int:image_id>", methods=["POST"])
@login_required
def set_featured_image(image_id):
    image = SubmissionImage.query.get_or_404(image_id)
    plant = image.plant
    if not plant:
        flash("This image is not linked to a plant.", "danger")
        return redirect(request.referrer or url_for("core_ui.home"))

    if current_user.id != plant.owner_id and current_user.role != "admin":
        flash("Not authorized to set featured image.", "danger")
        return redirect(request.referrer or url_for("core_ui.home"))

    FeaturedImage.query.filter_by(submission_image_id=image_id).delete()
    featured = FeaturedImage(submission_image_id=image_id, is_featured=True)
    db.session.add(featured)
    db.session.commit()
    flash("Image set as featured.", "success")
    return redirect(request.referrer or url_for("core_ui.home"))